﻿using System;
using System.Collections.Generic;
using System.Linq;
using System.Text;
using System.Windows;
using System.Windows.Controls;
using ESRI.ArcGIS.Client;
using ESRI.ArcGIS.Client.Geometry;

namespace GISforTT_API
{
    public class Bookmark
    {
        public enum BookmarkPriority
        {
            High = 1,
            Medium = 2,
            Low = 3,
        }

        #region members and properties

        private string id;
        public string ID
        {
            get
            {
                return id;
            }

            set
            {
                id = value;
            }
        }

        private string desc;
        public string Description
        {
            get
            {
                return desc;
            }
            set
            {
                desc = value;
            }
        }

        private BookmarkPriority priority = BookmarkPriority.Medium;
        public BookmarkPriority Priority
        {
            get
            {
                return priority;
            }
            set
            {
                priority = value;
            }
        }

        private MapPoint map_pt;
        public MapPoint MapCoordinate
        {
            get
            {
                return map_pt;
            }
        }

        private Point mf_center;
        public Point MapFrameCenter
        {
            get
            {
                return mf_center;
            }
            set
            {
                mf_center = value;
            }
        }

        private double mf_width;
        public double MapFrameWidth
        {
            get
            {
                return mf_width;
            }
            set
            {
                mf_width = value;
            }
        }

        private double mf_height;
        public double MapFrameHeight
        {
            get
            {
                return mf_height;
            }
            set
            {
                mf_height = value;
            }
        }

        private Envelope mf_env;
        public Envelope MapFrameEnvelope
        {
            get
            {
                return mf_env;
            }
            set
            {
                mf_env = value;
            }
        }

        private bool prev_open;
        public bool MapFramePreviouslyOpen
        {
            get
            {
                return prev_open;
            }
            set
            {
                prev_open = value;
            }
        }

        private bool current_open;
        public bool MapFrameCurrentlyOpen
        {
            get
            {
                return current_open;
            }
            set
            {
                current_open = value;
            }
        }

        private double orient;
        public double MapFrameOrientation
        {
            get
            {
                return orient;
            }
            set
            {
                orient = value;
            }
        }
        #endregion

        /// <summary>
        /// Constructor of the Bookmark object.
        /// </summary>
        /// <param name="location">MapPoint representing the geo-referenced location of the bookmark</param>
        /// <param name="description">Text description of the bookmark</param>
        /// <param name="Id">Unique ID representing the bookmark</param>
        /// <param name="p">Priority of the bookmark</param>
        public Bookmark(MapPoint location, string description, string Id, BookmarkPriority p)
        {
            id = Id;
            desc = description;
            priority = p;

            map_pt = location;
            mf_env = new Envelope(location.X - 0.01, location.Y - 0.01, location.X + 0.01, location.Y + 0.01);            
            mf_center = new Point(-1, -1);
            mf_height = 270;
            mf_width = 270;
            prev_open = false;
            current_open = false;
            orient = 0;

        }


        /// <summary>
        /// Constructor of the Bookmark object.
        /// </summary>
        /// <param name="envelope">Map envelope representing the location of the bookmark</param>
        /// <param name="description">Text description of the bookmark</param>
        /// <param name="Id">Unique ID representing the bookmark</param>
        /// <param name="p">Priority of the bookmark</param>
        public Bookmark(Envelope envelope, string description, string Id, BookmarkPriority p)
        {
            id = Id;
            desc = description;
            priority = p;

            map_pt = new MapPoint(envelope.XMin + ((envelope.XMax - envelope.XMin) / 2), envelope.YMin + ((envelope.YMax - envelope.YMin) / 2));
            mf_env = envelope;            
            mf_center = new Point(-1, -1);
            mf_height = 500;
            mf_width = 500;
            prev_open = false;
            current_open = false;
            orient = 0;
        }

        /// <summary>
        /// Constructor of the Bookmark object.
        /// </summary>
        /// <param name="xMin">Minimum x location (latitude)</param>
        /// <param name="yMin">Minimum y location (longitude)</param>
        /// <param name="xMax">Maximum x location</param>
        /// <param name="yMax">Maximum y location</param>
        /// <param name="description">Text description of the bookmark</param>
        /// <param name="Id">Unique ID representing the bookmark</param>
        /// <param name="p">Priority of the bookmark</param>
        public Bookmark(double xMin, double yMin, double xMax, double yMax, string description, string Id, BookmarkPriority p)
        {
            id = Id;
            desc = description;
            priority = p;

            map_pt = new MapPoint(xMin + ((xMax - xMin) / 2), yMin + ((yMax - yMin) / 2));
            mf_env = new Envelope(xMin, yMin, xMax, yMax);                        
            mf_center = new Point(-1, -1);
            mf_height = 270;
            mf_width = 270;
            prev_open = false;
            current_open = false;            
            orient = 0;
        }

        /// <summary>
        /// Return a string representing the priority of this bookmark.
        /// </summary>
        /// <returns>High, Meduim or Low</returns>
        public string getBookmarkPriorityString()
        {
            switch (priority)
            {
                case Bookmark.BookmarkPriority.High: { return "High"; }
                case Bookmark.BookmarkPriority.Medium: { return "Meduim"; }
                case Bookmark.BookmarkPriority.Low: { return "Low"; }
                default: { return string.Empty; }
            }
        }

    }
}
