﻿using System;
using System.Windows;
using System.Windows.Controls;
using System.Windows.Input;
using System.Windows.Media;
using System.Windows.Media.Imaging;
using System.Collections.Generic;

using ESRI.ArcGIS.Client;
using ESRI.ArcGIS.Client.Geometry;
using ESRI.ArcGIS.Client.Symbols;
using ESRI.ArcGIS.Client.Tasks;
using ESRI.ArcGIS.Client.Toolkit;

using Microsoft.Surface;
using Microsoft.Surface.Presentation;
using Microsoft.Surface.Presentation.Controls;
using Microsoft.Surface.Presentation.Input;


namespace GISforTT_API
{
    /// <summary>
    /// This class represents the scatterview item which will hold the map.
    /// It includes the frame of the map with the toolbar buttons and the actions of these buttons.
    /// Inside of the frame is the actual map control.
    /// </summary>
    public partial class MapFrame : ScatterViewItem
    {
        private GISforTT_API.Bookmark bookmark;
        private GhostFrame linkedGhostFrame;
        /*private int _id;        

        public int ID
        {
            get
            {
                return _id;
            }
        }*/

        public GISforTT_API.Bookmark RelatedBookmark
        {
            get
            {
                return bookmark;
            }
            set
            {
                bookmark = value;
            }
        }

        #region Constructors and Initializations

        /// <summary>
        /// Constructor of Map Frame
        /// </summary>        
        /// <param name="relatedGhostFrame">GhostFrame object corresponding to this map frame</param>
        /// <param name="p1">First map point - map location for the first corner of the map</param>
        /// <param name="p2">Second map point - map location for the second corner of the map</param>
        public MapFrame(GhostFrame relatedGhostFrame, MapPoint p1, MapPoint p2)
        {
            InitializeComponent();

            linkedGhostFrame = relatedGhostFrame;
            linkedGhostFrame.LinkedMapFrame = this;

            MainMap.Map.Loaded += new RoutedEventHandler(MainMap_Loaded);

            // Initialize the map
            InitializeMap();
            LoadMap(p1, p2);

            UpdateVisibilityOfAnnotationButtons();
            
        }


        /// <summary>
        /// Constructor for MapFrame
        /// </summary>
        /// <param name="relatedGhostFrame">GhostFrame object corresponding to this map frame</param>
        /// <param name="env">Map envelope representing the location of the map inside the frame</param>
        /// <param name="color">Color of the map frame</param>
        public MapFrame(GhostFrame relatedGhostFrame, Envelope env, SolidColorBrush color)
        {
            InitializeComponent();

            linkedGhostFrame = relatedGhostFrame;
            linkedGhostFrame.LinkedMapFrame = this;

            glassFrame1.Background = color;
            glassFrame2.Background = color;
            glassFrame3.Background = color;
            glassFrame4.Background = color;

            MainMap.Map.Loaded += new RoutedEventHandler(MainMap_Loaded);

            // Initialize the map
            InitializeMap();
            LoadMap(env);

            UpdateVisibilityOfAnnotationButtons();
        }

        /// <summary>
        /// Constructor of MapFrame
        /// </summary>
        /// <param name="relatedGhostFrame">GhostFrame object corresponding to this map frame</param>
        /// <param name="p1">First map point - map location for the first corner of the map</param>
        /// <param name="p2">Second map point - map location for the second corner of the map</param>
        /// <param name="color">Color of the map frame</param>
        public MapFrame(GhostFrame relatedGhostFrame, MapPoint p1, MapPoint p2, SolidColorBrush color)
        {
            InitializeComponent();

            linkedGhostFrame = relatedGhostFrame;
            linkedGhostFrame.LinkedMapFrame = this;

            glassFrame1.Background = color;
            glassFrame2.Background = color;
            glassFrame3.Background = color;
            glassFrame4.Background = color;

            MainMap.Map.Loaded += new RoutedEventHandler(MainMap_Loaded);

            // Initialize the map
            InitializeMap();
            LoadMap(p1, p2);

            UpdateVisibilityOfAnnotationButtons();
        }

        public void UpdateVisibilityOfAnnotationButtons()
        {
            MapLayersManager mlm = MapLayersManager.Instance;
            if (!mlm.annotationsLayerAdded && !mlm.onlineAnnotationLayersAdded)
            {
                // Hide and disable annotation buttons
                DrawPointButton.Visibility = Visibility.Hidden;
                DrawPointButton.IsEnabled = false;

                DrawPolylineButton.Visibility = Visibility.Hidden;
                DrawPolylineButton.IsEnabled = false;

                DrawPolygonButton.Visibility = Visibility.Hidden;
                DrawPolygonButton.IsEnabled = false;

                DrawRectangleButton.Visibility = Visibility.Hidden;
                DrawRectangleButton.IsEnabled = false;

                DrawFreehandButton.Visibility = Visibility.Hidden;
                DrawFreehandButton.IsEnabled = false;

                StopDrawButton.Visibility = Visibility.Hidden;
                StopDrawButton.IsEnabled = false;

                ClearGraphicsButton.Visibility = Visibility.Hidden;
                ClearGraphicsButton.IsEnabled = false;                   

            }
        }

        # endregion

/*        #region Delegations for Map Layer Changes

        /// <summary>
        /// Called automatically when a map layer is added in the map layer manager to update the contents of the map frame.
        /// </summary>
        /// <param name="addLayer">The added layer</param>
        public void MapLayerAdded(Layer addLayer)
        {
            Layer clone = MapLayersManager.Instance.CloneLayer(addLayer);
            if (clone != null)
                MainMap.AddMapLayer(clone);
        }

        /// <summary>
        /// Called automatically when a map layer is deleted from the map layer manager to update the contents of the map frame.
        /// </summary>
        /// <param name="deleteLayer">The deleted layer</param>
        public void MapLayerDeleted(Layer deleteLayer)
        {
            MainMap.RemoveMapLayer(deleteLayer);
        }

        #endregion*/

        #region Other Members

        /// <summary>
        /// Loads an image - for the buttons.
        /// </summary>
        /// <param name="path"></param>
        /// <returns></returns>
        private static Image LoadImageFromPath(string path)
        {
            // TODO: remove the redundant object
            ImageSourceConverter converter = new ImageSourceConverter();
            Image image = new Image();

            BitmapImage bitMapImage = new BitmapImage();
            bitMapImage.BeginInit();
            bitMapImage.UriSource = new Uri(path, UriKind.Relative);
            bitMapImage.EndInit();
            image.Stretch = Stretch.Fill;
            image.Source = bitMapImage;
            return image;
        }

        /// <summary>
        /// This button is supposed to save the changes done in the map.
        /// Currently it closes the frame and save the position of the frame to
        /// Bookmark object ONLY.        
        /// </summary>
        /// <param name="sender"></param>
        /// <param name="e"></param>
        private void ArchiveButton_Click(object sender, RoutedEventArgs e)
        {
            if (bookmark != null)
            {
                bookmark.MapFrameEnvelope = MainMap.Map.Extent;
                bookmark.MapFrameWidth = this.Width;
                bookmark.MapFrameHeight = this.Height;
                bookmark.MapFramePreviouslyOpen = true;
                bookmark.MapFrameCurrentlyOpen = false;
                bookmark.MapFrameCenter = this.Center;
                bookmark.MapFrameOrientation = this.Orientation;
            }

            BackgroundMapLayer.Instance.RemoveMap(this);
        }

        /// <summary>
        /// Set the name of the map frame
        /// </summary>
        /// <param name="name"></param>
        public void SetMapFrameName(string name)
        {
            MapFrameTitle.Text = name;
        }

        /// <summary>
        /// Returns the related Ghost Frame
        /// </summary>
        /// <returns></returns>
        public GhostFrame GetGhostFrame()
        {
            return linkedGhostFrame;
        }

        private void MapLayersButton_Click(object sender, RoutedEventArgs e)
        {
            if (MainMap.ShowLayersList)
            {
                MainMap.ShowLayersList = false;
            }
            else
            {
                MainMap.ShowLayersList = true;
            }

        }

        private void SaveButton_Click(object sender, RoutedEventArgs e)
        {
            if (bookmark != null)
            {
                bookmark.MapFrameEnvelope = MainMap.Map.Extent;
                bookmark.MapFrameWidth = this.Width;
                bookmark.MapFrameHeight = this.Height;
                bookmark.MapFramePreviouslyOpen = true;
                bookmark.MapFrameCurrentlyOpen = false;
                bookmark.MapFrameCenter = this.Center;
                bookmark.MapFrameOrientation = this.Orientation;
            }
        }

        private void RefreshMapButton_Click(object sender, RoutedEventArgs e)
        {
            MainMap.ReloadMap();
        }


        # endregion

        #region ArcGisMap Functions
        
        /// <summary>
        /// Initialize the map
        /// </summary>
        private void InitializeMap()
        {
            
            MainMap.LinkedGhostFrame = linkedGhostFrame;
            MainMap.MapWidth = MainMap.Width;
            MainMap.MapHeight = MainMap.Height;
            MainMap.ShowLayersList = false;            
        }


        private void LoadMap(Envelope env)
        {
            MainMap.setExtentFromEnvelop(env);
        }

        private void LoadMap(MapPoint p1, MapPoint p2)
        {
            Envelope env = new Envelope(p1, p2);
            MainMap.setExtentFromEnvelop(env);
            //MainMap.setExtentFromLtdLng(p1.Y, p1.X);
        }


        private void MainMap_Loaded(object sender, RoutedEventArgs e)
        {
            linkedGhostFrame.UpdatePosition(MainMap.Map.Extent);
        }

        # endregion
        
        
        #region scalling

        /// <summary>
        /// Changing the size of the map
        /// </summary>
        /// <param name="sender"></param>
        /// <param name="e"></param>
        private void ScatterViewItem_SizeChanged(object sender, SizeChangedEventArgs e)
        {
            if (e.PreviousSize.Height == 0)
                return;
                        
            // Resize the map grid
            MainMap.Width += e.NewSize.Width - e.PreviousSize.Width;
            MainMap.Height += e.NewSize.Height - e.PreviousSize.Height;
            linkedGhostFrame.UpdatePosition(MainMap.Extent);
        }

        # endregion scalling

        #region Map_Graphics
        
        private void DrawPointButton_Click(object sender, RoutedEventArgs e)
        {
            MainMap.SetDrawPointMode();
        }

        private void DrawPolylineButton_Click(object sender, RoutedEventArgs e)
        {
            MainMap.SetDrawPolylineMode();
        }

        private void DrawPolygonButton_Click(object sender, RoutedEventArgs e)
        {
            MainMap.SetDrawPolygonMode();
        }

        private void DrawRectangleButton_Click(object sender, RoutedEventArgs e)
        {
            MainMap.SetDrawRectangleMode();
        }

        private void DrawFreehandButton_Click(object sender, RoutedEventArgs e)
        {
            MainMap.SetDrawFreehandMode();
        }

        private void StopDrawButton_Click(object sender, RoutedEventArgs e)
        {
            MainMap.StopDrawing();
        }

        private void ClearGraphicsButton_Click(object sender, RoutedEventArgs e)
        {
            MainMap.ClearGraphics();
        }

        # endregion Map_Graphics

        #region Events on Map

        /// <summary>
        ///
        /// This handles the event when a touch lands on top
        /// of ArcGisMap not the scatterviewitem frame, we
        /// mark the event as handled to prevent it from
        /// bubbling up to the scatterviewitem such that
        /// scatterviewitem will not have a chance to 
        /// handle this touch event
        /// </summary>
        /// <param name="sender"></param>
        /// <param name="e"></param>
        private void MainMap_TouchDown(object sender, TouchEventArgs e)
        {
            if(sender is ArcGisMap) e.Handled = true;
        }

        /// <summary>
        /// This handles the touch event when the touch is moving
        /// on top of ArcGisMap, we mark this as handled so that
        /// the scaterviewitem container will not receive this
        /// event.
        /// </summary>
        /// <param name="sender"></param>
        /// <param name="e"></param>
        private void MainMap_TouchMove(object sender, TouchEventArgs e)
        {
            if(sender is ArcGisMap) e.Handled = true;
        }

        /// <summary>
        /// This handles the touch event when a touch is lifted
        /// on top of ArcGisMap, we mark it as handled so that
        /// the scatterviewitem container will not received this 
        /// event
        /// </summary>
        /// <param name="sender"></param>
        /// <param name="e"></param>
        private void MainMap_TouchUp(object sender, TouchEventArgs e)
        {
            if(sender is ArcGisMap) e.Handled = true;
        }

        # endregion

        /// <summary>
        /// Update the extent of the map
        /// </summary>
        /// <param name="mapEnv">new map envelope</param>
        public void UpdateMap(Envelope mapEnv)
        {
            MainMap.setExtentFromEnvelop(mapEnv);
        }

    }
}
